import climenu

@climenu.group(title='Build Menu')
def build_group(): pass


@build_group.menu(title='Build the package')
def build_package():
    # TODO: Add the actual code here
    print("I built the package!")

@build_group.menu(title='Build the release')
def build_release():
    '''
    Do a bunch of stuff to build the release.

        * Copy the files
        * Run some scripts
        * Build the release
        * Copy the release to X/Y/Z
    '''
    # TODO: Add the actual code here
    print("I built the release!")

def main():
    climenu.run()

if __name__ == '__main__':
    main()


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
This is the "test" submenu.
'''
import climenu

@climenu.group(title='Test Menu')
def test_menu(): pass

@test_menu.menu(title='Run test #1')
def test_one():
    pass

@test_menu.menu(title='Run test #2')
def test_two():
    pass

@test_menu.menu(title='Run test #3')
def test_three():
    pass

@test_menu.menu(title='Run test #4')
def test_four():
    pass


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
This sample script shows a simple **nested** menu structure.

There are two `group` items: `build_menu` and `test_menu`.  These two items
are shown on the main menu.  Once selected, thier respective sub-items will
be shown.

`test_menu` has a sub-menu itself, `sub_test_menu`.  That item has a single
menu item.

Notice how the decorators change from `@climenu` to `@build_menu` and
`@test_menu`.
'''
from __future__ import print_function
import climenu


###############################################################################
# Create an empty function to serve as a menu group
@climenu.group()
def build_menu():
    '''Build Functions'''
    pass


# Add this function to the `build_menu` group
@build_menu.menu()
def test():
    '''Build the package'''
    print("!!!package build!!!")
    return True


# Add this function to the `build_menu` group
@build_menu.menu()
def test2():
    '''Build the release'''
    print("!!!release build")
    return True
###############################################################################

###############################################################################
# Create an empty function to serve as a menu group
@climenu.group()
def test_menu():
    '''Test Functions'''
    pass


# Add this function to the `test_menu` group
@test_menu.menu()
def test_one():
    '''Run test #1'''
    print("!!!test #1 run!!!")
    return True


# Add this function to the `test_menu` group
@test_menu.menu()
def test_two():
    '''Run test #2'''
    print("!!!test #2 run!!!")


# Create a sub-group and add it to the `test_menu` group
@test_menu.group()
def sub_test_menu():
    '''Another testing menu'''
    pass


# Add this function to the `subsub_menu` group
@sub_test_menu.menu()
def subsub_menu1():
    '''Run test #3'''
    print("!!!test #3 run!!!")
#################################################################################

if __name__ == '__main__':
    climenu.run()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
This sample script shows a simple **flat** menu structure.  Each menu item
will be listed on the "main menu".

We also modify some of `climenu`'s settings.
'''
from __future__ import print_function
import climenu


# Let the user press the enter key to exit the application
climenu.settings.back_values = ['0', '']
climenu.settings.text['main_menu_title'] = climenu.colors.red('Application AAA Management')


@climenu.menu(title=climenu.colors.blue('Build the package'))
def build_package():
    '''Build the package'''
    print("!!!package built!!!")
    return True


@climenu.menu(title=climenu.colors.yellow('Build the release', bright=True))
def build_releasea():
    '''Build the release'''
    print("!!!release built!!!")
    return True


@climenu.menu(title=climenu.colors.magenta('Run test #1'))
def test_1():
    '''Run test #1'''
    print("!!!test #1 pass!!!")
    return True


# Here's an example of the menu title being different from the function
# docstring.
@climenu.menu(title=climenu.colors.cyan('Run test #2'))
def test_2():
    '''This function tests functionality 1.2.3'''
    print("!!!test #2 run!!!")


if __name__ == '__main__':
    climenu.run()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
This is the "build" submenu.
'''
import climenu

@climenu.group(title='Build Menu')
def build_menu(): pass

@build_menu.menu(title='Build package')
def build_package():
    pass

@build_menu.menu(title='Build release')
def build_release():
    pass



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
Short example of ``climenu``
'''
from __future__ import print_function
import climenu


@climenu.menu()
def build_packages():
    '''Build packages'''
    # TODO: Call the build scripts here!
    print('built the packages!')

@climenu.menu()
def build_release():
    '''Build release'''
    # TODO: Call the build scripts here!
    print('built the release!')


if __name__ == '__main__':
    climenu.run()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
'''
This is a menu-driven command line application to
facilitate building and testing.
'''

import build_menu  # pylint: disable=W0611
import test_menu  # pylint: disable=W0611
import climenu


if __name__ == '__main__':
    climenu.run()


import climenu

@climenu.group()
def build_group():
    '''Build Menu'''
    pass

@build_group.menu()
def build_package():
    '''Build the package'''
    # TODO: Add the actual code here
    print("I built the package!")

@build_group.menu()
def build_release():
    '''Build the release'''
    # TODO: Add the actual code here
    print("I built the release!")

def main():
    climenu.run()

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
climenu setup script
'''
import re
from setuptools import setup


MAIN_NS = {}
VER_PATH = 'climenu.py'


def get_version():
    re_ver = re.compile(
        '^__version__\s*=\s+[\'"](?P<version>.*?)[\'"]', re.MULTILINE)
    text = open(VER_PATH).read()
    return re_ver.search(text).group('version')


if __name__ == '__main__':
    DESC = open('README.rst').read()
    version = get_version()

    setup(
        name="climenu",
        py_modules=['climenu'],
        version=version,
        description="Command-line menu system",
        url="https://github.com/mtik00/pyclimenu",
        download_url=(
            "https://github.com/mtik00/pyclimenu/releases/download/v{0}"
            "/climenu-{0}.tar.gz").format(version),

        keywords='cli command-line menu',

        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            'Environment :: Console',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 3',
            'Development Status :: 5 - Production/Stable'
        ],

        author="Timothy McFadden",
        author_email="tim@timandjamie.com",
        license='MIT',
        install_requires=[],
        zip_safe=True,
        include_package_data=True,
        long_description=DESC
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
This module is used to create *simple* menu-based CLI programs in Python.

This package is **highly** inspired by Click (http://click.pocoo.org).
'''

# Imports #####################################################################
from __future__ import print_function
import os
import sys

# Metadata ####################################################################
__author__ = 'Timothy McFadden'
__creationDate__ = '06-APR-2017'
__version__ = '1.2.0'


# Globals #####################################################################
__all__ = [
    'menu', 'group', 'settings', 'colors',
]
(IS_WIN, IS_LIN) = ('win' in sys.platform, 'lin' in sys.platform)
MENU_ITEMS = []


class Settings(object):
    '''
    This class is used to store the settings for ``climenu``.
    '''
    clear_screen = True
    text = {
        'main_menu_title': 'Main Menu',
        'main_menu_prompt': 'Enter the selection ([{q}] to quit): ',
        'submenu_prompt': 'Enter the selection ([{back}] to return, {q} to quit): ',
        'invalid_selection': 'Invalid selection.  Please try again. ',
        'continue': 'Press Enter to continue: ',
    }

    # Add ``''`` to this list to go back one level if the user doesn't
    # enter anything
    back_values = ['0']

    # Change this to the character or phrase that will immediately exit the
    # menu.
    quit_value = 'q'

    # Set this to true if you are using colors but need to disable them
    # (e.g. a platform you use doesn't support it)
    disable_colors = False

    def get_submenu_prompt(self):
        return self.text['submenu_prompt'].format(
            back=", ".join([x or '""' for x in self.back_values]),
            q=self.quit_value
        )

    def get_main_menu_prompt(self):
        return self.text['main_menu_prompt'].format(
            q=", ".join([x or '""' for x in self.back_values + [self.quit_value]]),
            # q=self.quit_value
        )


settings = Settings()  # pylint: disable=C0103


def _show_main_menu(menu_items):
    '''Show the main menu and return the selected item.'''

    while True:
        print(settings.text['main_menu_title'])

        for index, menu_group in enumerate(menu_items):
            print("%2i : %s" % (index + 1, menu_group.title))

        print()
        value = get_user_input(settings.get_main_menu_prompt())

        if value.lower() in settings.back_values + [settings.quit_value]:
            return None

        if not(value.isdigit()) or (int(value) <= 0) or (int(value) > len(menu_items)):
            print(settings.text['invalid_selection'])
            continue

        return menu_items[int(value) - 1]


def _show_group_menu(menu_group):
    '''Show a submenu and return the selected item.'''
    while True:
        print(menu_group.title)

        submenu_items = menu_group.menus
        for index, submenu in enumerate(submenu_items):
            print("%2i : %s" % (index + 1, submenu.title))

        print()
        value = get_user_input(settings.get_submenu_prompt())

        if value in settings.back_values:
            return None
        elif value.lower() == settings.quit_value:
            sys.exit(0)

        if not(value.isdigit()) or (int(value) > len(submenu_items)):
            print(settings.text['invalid_selection'])
            continue

        return submenu_items[int(value) - 1]


def run():
    '''
    Runs the menuing system.
    '''
    menu_stack = []
    current_group = None

    if not MENU_ITEMS:
        raise ValueError("No menu items defined")

    while True:
        # Clear the screen in-between each menu
        if settings.clear_screen:
            clear_screen()

        if not current_group:
            menu_item = _show_main_menu(MENU_ITEMS)
            if not menu_item:
                break
        else:
            menu_item = _show_group_menu(current_group)

        if (not menu_item) and menu_stack:
            back_one = menu_stack.pop()
            if back_one != current_group:
                current_group = back_one
            elif menu_stack:
                # Pop another one off
                current_group = menu_stack.pop()
            else:
                # Show the main menu (nothing left in the stack)
                current_group = None
            continue

        # Check for a sub-menu.  Sub-menu's don't
        # have a callback, so just set the current
        # group and loop.
        if isinstance(menu_item, MenuGroup):
            menu_stack.append(menu_item)
            current_group = menu_item
            continue

        # If we should show the *main* menu, then
        # ``menu_item`` will be None here.
        if menu_item:
            menu_item.callback()
            get_user_input(settings.text['continue'])
        else:
            # Nothing left in the stack; make
            # ``current_group == None`` so we'll
            # show the main menu the next time through
            # the loop.
            current_group = None


def clear_screen():
    '''Clears the terminal window'''
    if IS_WIN:
        os.system('cls')
    elif IS_LIN:
        os.system('clear')
    else:
        raise NotImplementedError(
            "Your platform has not been implemented: %s" % sys.platform)


def get_user_input(prompt=None, test_value=None):
    '''
    Prompt the user for input.

    :param str prompt: The text to show the user
    :param var test_value: If this is not none, the user will not be prompted
        and this value is returned.
    '''
    if prompt:
        print(prompt, end='')

    if test_value is not None:
        return test_value

    if sys.version_info.major == 2:
        return raw_input()
    else:
        return input()


class Menu(object):
    '''A sinlge menu item with a callback'''
    def __init__(self, title, callback):
        self.callback = callback
        self.title = title


class MenuGroup(object):
    '''A group of Menu items'''
    def __init__(self, title, menus=None):
        self.title = title
        self.menus = menus or []

    def menu(self, *args, **kwargs):  # pylint: disable=W0613
        '''Decorator to add a menu item to our list'''
        def decorator(decorated_function):
            '''create a menu item decorator'''
            menu_ = Menu(
                kwargs.get('title') or decorated_function.__doc__,
                callback=decorated_function)
            self.menus.append(menu_)
            return menu_
        return decorator

    def group(self, *args, **kwargs):  # pylint: disable=W0613
        '''Decorator to add a menu group to our list'''
        def decorator(decorated_function):
            '''create a menu group decorator'''
            menu_ = MenuGroup(
                kwargs.get('title') or decorated_function.__doc__)
            self.menus.append(menu_)
            return menu_
        return decorator


def group(title=None):
    '''A decorator to create a new MenuGroup'''
    def decorator(decorated_function):
        '''create a menu group decorator'''
        group_ = MenuGroup(title or decorated_function.__doc__)
        MENU_ITEMS.append(group_)
        return group_
    return decorator


def menu(title=None):
    '''A decorator to create a single menu item'''
    def decorator(decorated_function):
        '''create a menu item decorator'''
        menu_ = Menu(
            title or decorated_function.__doc__, callback=decorated_function)
        MENU_ITEMS.append(menu_)
        return menu_
    return decorator


class ANSIColors(object):
    '''
    A class to format strings with ANSI color codes.

    The codes available are only the "first 8" color codes.  See here for
    more information:
    https://en.wikipedia.org/wiki/ANSI_escape_code#Colors
    '''
    (black_code, red_code, green_code, yellow_code, blue_code,
     magenta_code, cyan_code, white_code) = range(8)
    fg_code = 3
    bg_code = 4

    def black(self, text, bg=False, bright=False):
        return self._wrap(text, self.black_code, bg=bg, bright=bright)

    def red(self, text, bg=False, bright=False):
        return self._wrap(text, self.red_code, bg=bg, bright=bright)

    def green(self, text, bg=False, bright=False):
        return self._wrap(text, self.green_code, bg=bg, bright=bright)

    def yellow(self, text, bg=False, bright=False):
        return self._wrap(text, self.yellow_code, bg=bg, bright=bright)

    def blue(self, text, bg=False, bright=False):
        return self._wrap(text, self.blue_code, bg=bg, bright=bright)

    def magenta(self, text, bg=False, bright=False):
        return self._wrap(text, self.magenta_code, bg=bg, bright=bright)

    def cyan(self, text, bg=False, bright=False):
        return self._wrap(text, self.cyan_code, bg=bg, bright=bright)

    def white(self, text, bg=False, bright=False):
        return self._wrap(text, self.white_code, bg=bg, bright=bright)

    def _wrap(self, text, color_code, bg=False, bright=False):
        if settings.disable_colors:
            return text

        return "\033[{fgbg}{code}{bright}m{text}\033[0m".format(
            fgbg=self.bg_code if bg else self.fg_code,
            code=color_code,
            text=text,
            bright=";1" if bright else ""
        )


colors = ANSIColors()
